"""
Nuqql Conversations
"""

import curses
import datetime
import logging

from types import SimpleNamespace
from typing import TYPE_CHECKING, Dict, List, Optional, Tuple

import nuqql.config

from nuqql.win import InputWin, LogWin
from .history import History
from .logmessage import LogMessage

if TYPE_CHECKING:   # imports for typing
    # pylint: disable=cyclic-import
    # pylint hack: avoid code-duplication warning with buddyconversation
    # (see: https://github.com/PyCQA/pylint/issues/214)
    from nuqql.account import Account  # noqa
    from nuqql.backend import Backend  # noqa
    from nuqql.buddy import Buddy   # noqa

logger = logging.getLogger(__name__)

# list of active conversations
CONVERSATIONS: List["Conversation"] = []


class Conversation:
    """
    Class for conversations
    """

    def __init__(self, backend: Optional["Backend"], account:
                 Optional["Account"], name: str) -> None:
        # general
        self.name = name
        self.notification = 0

        # statistics
        self.stats: Dict[str, float] = {}
        self.stats["last_used"] = 0
        self.stats["last_send"] = 0
        self.stats["num_send"] = 0

        # backend info
        self.backend = backend
        self.account = account

        # windows of the conversation
        self.wins = SimpleNamespace()
        self.wins.log_win = None
        self.wins.input_win = None
        self.wins.list_win = None

        # history and logging
        self.history = History(self)

    def activate(self, set_last_used: bool = True) -> None:
        """
        Activate windows of conversation
        """

        # check log_win to determine, if windows are already created
        if self.wins.log_win is not None:
            curses.curs_set(1)
            self.wins.input_win.state.active = True
            self.wins.input_win.redraw()
            self.wins.log_win.state.active = False
            self.wins.log_win.redraw()
            self.clear_notifications()
            if set_last_used:
                self.stats["last_used"] = datetime.datetime.now().timestamp()
            logger.debug("activated conversation %s", self.name)
            return

    def activate_log(self) -> None:
        """
        Activate windows of conversation and go to history
        """

        # check log_win to determine, if windows are already created
        if self.wins.log_win is not None:
            curses.curs_set(1)
            self.wins.input_win.state.active = False
            self.wins.log_win.state.active = True
            self.wins.log_win.zoom_win()
            self.clear_notifications()
            logger.debug("activated log of conversation %s", self.name)
            return

    def has_windows(self) -> bool:
        """
        Check if conversation has already created its windows
        """

        if self.wins.log_win:
            return True

        return False

    def _create_windows_common(self, log_title, input_title) -> None:
        """
        Helper for creating common windows
        """

        log_config = nuqql.config.get("log_win")
        self.wins.log_win = LogWin(log_config, self, log_title)
        self.wins.log_win.list = self.history.log
        input_config = nuqql.config.get("input_win")
        self.wins.input_win = InputWin(input_config, self, input_title)

    def create_windows(self) -> None:
        """
        Create windows for this conversation
        """

        # implemented in sub classes

    def _get_name_notification(self) -> str:
        """
        get notification prefix for name
        """

        # check if there are pending notifications
        if self.notification > 0:
            notify = "# "
        else:
            notify = ""

        return notify

    def get_name(self) -> str:
        """
        Get the name of the conversation, depending on type
        """

        # implemented in sub classes

    def log(self, sender: str, msg: str, tstamp: datetime.datetime = None,
            own: bool = False) -> LogMessage:
        """
        Log message to conversation's history/log window
        """

        logger.debug("logging message in conversation %s: "
                     "sender %s, timestamp %s, msg %s",
                     self.name, sender, tstamp, msg)

        # create a log message
        if tstamp is None:
            tstamp = datetime.datetime.now()
        log_msg = LogMessage(tstamp, sender, msg, own=own)

        # if conversation has not been initialized yet, stop here.
        # Messages must be loaded from the history file first
        if not self.wins.log_win:
            return log_msg

        # put message into conversation's history
        if self.history.log:
            last_msg = self.history.log[-1]
            if last_msg.tstamp.date() != tstamp.date():
                date_change_msg = LogMessage(
                    log_msg.tstamp, "<event>", "<Date changed to {}>".format(
                        log_msg.tstamp.date()), own=True)
                date_change_msg.is_read = True
                self.history.log.append(date_change_msg)
        self.history.log.append(log_msg)

        # if conversation is already active, redraw the log
        if self.is_input_win_active():
            self.wins.log_win.redraw_pad()
            self.wins.input_win.redraw_pad()    # keep cursor in input_win

        return log_msg

    def notify(self) -> None:
        """
        Notify this conversation about new messages
        """

        self.notification = 1

        if self.wins.list_win:
            self.wins.list_win.redraw_pad()

    def clear_notifications(self) -> None:
        """
        Clear notifications of buddy
        """

        self.notification = 0
        if self.wins.list_win:
            self.wins.list_win.redraw_pad()

    def __lt__(self, other: "Conversation"):
        # sort based on get_key output
        return self.get_key() < other.get_key()

    # status to sorting key mapping
    status_key = {
        "on": 0,
        "afk": 1,
        "grp": 2,
        "grp_invite": 3,
        "off": 4,
    }

    def get_key(self) -> Tuple:
        """
        Get a key for sorting this conversation
        """

        # implemented in sub classes

    def is_log_win_active(self) -> bool:
        """
        Check if conversation's log window is active
        """

        # check if log win is active
        if self.wins.log_win and self.wins.log_win.state.active:
            return True

        return False

    def is_input_win_active(self) -> bool:
        """
        Check if conversation's input window is active
        """

        # check if input win is active
        if self.wins.input_win and self.wins.input_win.state.active:
            return True

        return False

    def is_active(self) -> bool:
        """
        Check if this conversation is currently active, and return True if it
        is the case; otherwise, return False.
        """

        # check if input win and/or log win is active
        if self.is_input_win_active() or self.is_log_win_active():
            return True

        return False

    def is_any_active(self) -> bool:
        """
        Check if any conversation is currently active
        """

        if self.is_active():
            return True

        for conv in CONVERSATIONS:
            if conv.is_active():
                return True

        return False

    def get_new(self) -> Optional["Conversation"]:
        """
        Check if there is any conversation with new messages and return it
        """

        for conv in CONVERSATIONS:
            if conv.notification > 0:
                logger.debug("found new conversation in conversation %s: %s",
                             self.name, conv.name)
                return conv

        return None

    def get_next(self) -> Optional["Conversation"]:
        """
        Check if there is any newer used conversation and return it
        """

        next_conv = None
        for conv in CONVERSATIONS:
            if conv.stats["last_used"] == 0:
                continue
            if conv.stats["last_used"] > self.stats["last_used"]:
                if next_conv and \
                   next_conv.stats["last_used"] < conv.stats["last_used"]:
                    # already found a next newer conversation
                    continue
                next_conv = conv
                logger.debug("found next conversation in conversation %s: %s",
                             self.name, next_conv.name)

        return next_conv

    def get_prev(self) -> Optional["Conversation"]:
        """
        Check if there is any previously used conversation and return it
        """

        prev_conv = None
        for conv in CONVERSATIONS:
            if conv.stats["last_used"] == 0:
                continue
            if conv.stats["last_used"] < self.stats["last_used"]:
                if prev_conv and \
                   prev_conv.stats["last_used"] > conv.stats["last_used"]:
                    # already found a next older conversation
                    continue
                prev_conv = conv
                logger.debug("found previous conversation in conversation %s: "
                             "%s", self.name, prev_conv.name)

        return prev_conv

    def process_input(self, char: str) -> None:
        """
        Process user input in active window
        """

        # try to give control to the input win first...
        if self.wins.input_win and self.wins.input_win.state.active:
            self.wins.input_win.process_input(char)
            return

        # then, try to give control to the log win
        if self.wins.log_win and self.wins.log_win.state.active:
            self.wins.log_win.process_input(char)
            return

    def _send_msg_prepare(self, msg) -> None:
        """
        helper for running common operations when sending a message
        """

        # TODO: unify the logging in a method of Conversation?
        # log message
        tstamp = datetime.datetime.now()
        log_msg = LogMessage(tstamp, "you", msg, own=True)
        self.wins.log_win.add(log_msg)

        # statistics
        self.stats["last_send"] = datetime.datetime.now().timestamp()
        self.stats["num_send"] += 1

        # redraw list_win in case sorting is affected by stats update above
        self.wins.list_win.redraw_pad()

    def send_msg(self, msg: str) -> None:
        """
        Send message coming from the UI/input window
        """

        # implemented in sub classes

    def set_lastread(self) -> None:
        """
        Helper that sets lastread to the last message in the conversation,
        thus, marking all messages as read.
        """

        # only relevant for BuddyConversation. Implemented there.



"""
Nuqql UI Windows
"""

import curses
import logging

from types import SimpleNamespace
from typing import TYPE_CHECKING, Any, Callable, Dict, List, Tuple

if TYPE_CHECKING:   # imports for typing
    # pylint: disable=cyclic-import
    from nuqql.config import WinConfig  # noqa
    from nuqql.conversation import Conversation  # noqa

# screen and main windows
MAIN_WINS: Dict[str, Any] = {}

logger = logging.getLogger(__name__)


class Win:
    """
    Base class for Windows
    """

    def __init__(self, config: "WinConfig", conversation: "Conversation",
                 title: str) -> None:
        # configuration
        self.config = config

        # conversation
        self.conversation = conversation

        # window title
        self.title = " " + title + " "

        # create new window and new pad
        size_y, size_x = self.config.get_size()
        pos_y, pos_x = self.config.get_pos()
        self.win = curses.newwin(size_y, size_x, pos_y, pos_x)
        self.pad = curses.newpad(size_y - 2, size_x - 2)

        # window state
        self.state = SimpleNamespace(
            # is window zoomed?
            zoomed=False,
            # is window visible?
            visible=True,
            # is window active?
            active=False,
            # position inside pad
            pad_y=0,
            pad_x=0,
            # cursor positions
            cur_y=0,
            cur_x=0
        )

        # keymaps/bindings
        self.keyfunc: Dict[str, Callable] = {}
        self._init_keyfunc()

    def list_add(self, internal_list: List, entry: Any) -> None:
        """
        Add entry to internal list
        """

        # add entry to own list
        logger.debug("adding entry %s to internal list", entry)
        internal_list.append(entry)

        # if terminal size is invalid, stop here
        if not self.config.is_terminal_valid():
            return

        # if this window belongs to an active conversation, redraw it
        if self.conversation.is_active():
            self.redraw()
        elif self is MAIN_WINS["log"]:
            # if this is the main log, display it anyway if there is nothing
            # else active
            if self.conversation.is_any_active():
                return
            self.redraw()

    def _redraw_win(self) -> None:
        """
        Redraw entire window
        """

        # if terminal size is invalid, stop here
        if not self.config.is_terminal_valid():
            return

        # screen/window properties
        win_size_y, win_size_x = self.win.getmaxyx()
        self.win.erase()

        # color settings on
        self.win.attrset(self.config.attr["win_border"])

        # window border
        max_y, max_x = MAIN_WINS["screen"].getmaxyx()
        if not win_size_y == max_y or not win_size_x == max_x:
            self.win.border()

        # window title
        max_title_len = min(len(self.title), win_size_x - 3)
        title = self.title[:max_title_len]
        if title != "":
            title = title[:-1] + " "
        if self.config.settings["show_title"]:
            self.win.addstr(0, 2, title)

        self.win.refresh()

    def _get_pad_rect_size(self) -> Tuple[int, int]:
        """
        Get the y and x size of the visible pad rectangle
        """

        win_size_y, win_size_x = self.win.getmaxyx()
        pad_rect_y, pad_rect_x = win_size_y - 2, win_size_x - 2
        if self.state.zoomed:
            pad_rect_x = win_size_x
        return pad_rect_y, pad_rect_x

    def _move_pad(self) -> None:
        """
        Move the pad
        """

        # get size of pad rectangle
        pad_rect_y, pad_rect_x = self._get_pad_rect_size()

        # get current cursor positions
        self.state.cur_y, self.state.cur_x = self.pad.getyx()

        # move pad right (1/3 of the window's width), if cursor leaves window
        # area on the right
        if self.state.cur_x > self.state.pad_x + (pad_rect_x - 1):
            self.state.pad_x = self.state.cur_x - (pad_rect_x - 1) + \
                int(pad_rect_x/3)

        # move pad left (1/3 of the window's width), if cursor leaves current
        # pad position on the left
        if self.state.cur_x < self.state.pad_x:
            self.state.pad_x = self.state.cur_x - int(pad_rect_x/3)

        # move pad down, if cursor leaves window area at the bottom
        if self.state.cur_y > self.state.pad_y + (pad_rect_y - 1):
            self.state.pad_y = self.state.cur_y - (pad_rect_y - 1)

        # move pad up, if cursor leaves current pad position at the top
        if self.state.cur_y < self.state.pad_y:
            self.state.pad_y = self.state.cur_y

    def _check_borders(self) -> None:
        """
        Check borders
        """

        # get sizes
        pad_size_y, pad_size_x = self.pad.getmaxyx()
        pad_rect_y, pad_rect_x = self._get_pad_rect_size()

        # do not move visible area too far to the left
        if self.state.pad_x < 0:
            self.state.pad_x = 0

        # do not move visible area too far to the right
        if self.state.pad_x + pad_rect_x > pad_size_x:
            self.state.pad_x = pad_size_x - pad_rect_x

        # do not move visible area too far up
        if self.state.pad_y < 0:
            self.state.pad_y = 0

        # do not move visible area too far down
        if self.state.pad_y + pad_rect_y > pad_size_y:
            self.state.pad_y = pad_size_y - pad_rect_y

    def redraw_pad(self) -> None:
        """
        Redraw pad in window
        """

        # implemented in other classes

    def redraw(self) -> None:
        """
        Redraw the window
        """

        if self.state.visible:
            self._redraw_win()
            self.redraw_pad()

    def resize_win(self, win_y_max: int, win_x_max: int) -> None:
        """
        Resize window
        """

        # if terminal size is invalid, stop here
        if not self.config.is_terminal_valid():
            return

        logger.debug("resizing window to y = %d and x = %d",
                     win_y_max, win_x_max)
        # TODO: change function parameters?
        self.win.resize(win_y_max, win_x_max)

    def move_win(self, pos_y: int, pos_x: int) -> None:
        """
        Move window
        """

        # if terminal size is invalid, stop here
        if not self.config.is_terminal_valid():
            return

        logger.debug("moving window to y = %d and x = %d",
                     pos_y, pos_x)
        self.win.mvwin(pos_y, pos_x)

    def go_back(self, *args: Any) -> None:
        """
        User input: go back
        """

        # implemented in sub classes

    def _cursor_right(self, *args: Any) -> None:
        """
        User input: cursor right
        """

        # implemented in sub classes

    def _cursor_left(self, *args: Any) -> None:
        """
        User input: cursor left
        """

        # implemented in sub classes

    def _cursor_down(self, *args: Any) -> None:
        """
        User input: cursor down
        """

        # implemented in sub classes

    def _cursor_up(self, *args: Any) -> None:
        """
        User input: cursor up
        """

        # implemented in sub classes

    def _cursor_top(self, *args: Any) -> None:
        """
        User input: cursor top
        """

        # implemented in sub classes

    def _cursor_bottom(self, *args: Any) -> None:
        """
        User input: cursor bottom
        """

        # implemented in sub classes

    def _cursor_page_up(self, *args: Any) -> None:
        """
        User input: cursor page up
        """

        # implemented in sub classes

    def _cursor_page_down(self, *args: Any) -> None:
        """
        User input: cursor page down
        """

        # implemented in sub classes

    def _send_msg(self, *args: Any) -> None:
        """
        User input: send message
        """

        # implemented in sub classes

    def _delete_char(self, *args: Any) -> None:
        """
        User input: delete character
        """

        # implemented in sub classes

    def _delete_char_right(self, *args: Any) -> None:
        """
        User input: delete character on the right of current cursor position
        """

        # implemented in sub classes

    def _cursor_msg_start(self, *args: Any) -> None:
        """
        User input: move cursor to message start
        """

        # implemented in sub classes

    def _cursor_msg_end(self, *args: Any) -> None:
        """
        User input: move cursor to message end
        """

        # implemented in sub classes

    def _cursor_line_start(self, *args: Any) -> None:
        """
        User input: move cursor to line start
        """

        # implemented in sub classes

    def _cursor_line_end(self, *args: Any) -> None:
        """
        User input: move cursor to line end
        """

        # implemented in sub classes

    def _delete_line_end(self, *args: Any) -> None:
        """
        User input: delete from cursor to end of current line
        """

        # implemented in sub classes

    def _delete_line(self, *args: Any) -> None:
        """
        User input: delete current line
        """

        # implemented in sub classes

    def _go_log_search_url(self, *args: Any) -> None:
        """
        User input: go to conversation log/history and search for a URL
        """

        # implemented in sub classes

    def _go_log(self, *args: Any) -> None:
        """
        User input: go to the conversations log/history
        """

        # implemented in sub classes

    def _go_next(self, *args: Any) -> None:
        """
        User input: go to conversation with new messages or more recently used
        conversation
        """

        # implemented in sub classes

    def _go_prev(self, *args: Any) -> None:
        """
        User input: go to previously used conversation
        """

        # implemented in sub classes

    def _go_conv(self, *args: Any) -> None:
        """
        User input: go to specific conversation
        """

        # implemented in sub classes

    def _search(self, *args: Any) -> None:
        """
        User input: search
        """

        # implemented in sub classes

    def _search_next(self, *args: Any) -> None:
        """
        User input: search for next match
        """

        # implemented in sub classes

    def _search_prev(self, *args: Any) -> None:
        """
        User input: search for previous match
        """

        # implemented in sub classes

    def _enter(self, *args: Any) -> None:
        """
        User input: enter
        """

        # implemented in sub classes

    def _tab(self, *args: Any) -> None:
        """
        User input: tab
        """

        # implemented in sub classes

    def _quit(self, *args: Any) -> None:
        """
        User input: quit nuqql
        """

        # implemented in sub classes

    def handle_keybinds(self, *args: Any, keybinds: Dict[str, str] = None,
                        keyfunc: Dict[str, Callable] = None) -> bool:
        """
        Handle special keys
        """

        # input character is first argument
        char = args[0]

        # set keybinds and keyfunc, if not in parameters
        if not keybinds:
            keybinds = self.config.keybinds
        if not keyfunc:
            keyfunc = self.keyfunc

        # look for special key mappings in keymap
        try:
            cint = ord(char)
        except (TypeError, ValueError):
            cint = char
        if cint in self.config.keymap and \
           self.config.keymap[cint] in keybinds:
            logger.debug("handling keybind %d", cint)
            func = keyfunc[keybinds[self.config.keymap[cint]]]
            func(*args[1:])     # call function with remaining arguments
            return True         # handled a special key

        # no special key
        return False

    def _init_keyfunc(self) -> None:
        """
        Initialize key to function mapping
        """

        logger.debug("initializing key to function mapping")
        self.keyfunc = {
            "CURSOR_RIGHT": self._cursor_right,
            "CURSOR_LEFT": self._cursor_left,
            "CURSOR_DOWN": self._cursor_down,
            "CURSOR_UP": self._cursor_up,
            "CURSOR_TOP": self._cursor_top,
            "CURSOR_BOTTOM": self._cursor_bottom,
            "CURSOR_PAGE_UP": self._cursor_page_up,
            "CURSOR_PAGE_DOWN": self._cursor_page_down,
            "CURSOR_MSG_START": self._cursor_msg_start,
            "CURSOR_MSG_END": self._cursor_msg_end,
            "CURSOR_LINE_START": self._cursor_line_start,
            "CURSOR_LINE_END": self._cursor_line_end,
            "DEL_CHAR": self._delete_char,
            "DEL_CHAR_RIGHT": self._delete_char_right,
            "DEL_LINE_END": self._delete_line_end,
            "DEL_LINE": self._delete_line,
            "ENTER": self._enter,
            "GO_BACK": self.go_back,
            "GO_NEXT": self._go_next,
            "GO_PREV": self._go_prev,
            "GO_CONV": self._go_conv,
            "GO_LOG": self._go_log,
            "GO_LOG_SEARCH_URL": self._go_log_search_url,
            "QUIT": self._quit,
            "SEARCH": self._search,
            "SEARCH_NEXT": self._search_next,
            "SEARCH_PREV": self._search_prev,
            "SEND_MSG": self._send_msg,
            "TAB": self._tab,
        }



"""
Nuqql's User Interface configuration
"""

import argparse
import logging
import os
import stat

from pathlib import Path
from typing import Any, Dict

from nuqql import VERSION
from .configs import CONFIGS, get, read_from_file, write_to_file
from .winconfig import WinConfig

logger = logging.getLogger(__name__)

# default conversation settings
DEFAULT_CONVERSATION_CONFIG = {
    "sort_key": "last_send",
}


def init_win(screen: Any) -> None:
    """
    Initialize window configurations
    """

    list_win = WinConfig("list_win")
    log_win = WinConfig("log_win")
    log_win_main = WinConfig("log_win_main")
    input_win = WinConfig("input_win")

    list_win.init_layout()
    log_win.init_layout()
    log_win_main.init_layout()
    input_win.init_layout()

    list_win.init_window_settings()
    log_win.init_window_settings()
    log_win_main.init_window_settings()
    input_win.init_window_settings()

    list_win.init_colors()
    log_win.init_colors()
    log_win_main.init_colors()
    input_win.init_colors()

    list_win.init_keymap()
    log_win.init_keymap()
    log_win_main.init_keymap()
    input_win.init_keymap()

    list_win.init_keybinds()
    log_win.init_keybinds()
    log_win_main.init_keybinds()
    input_win.init_keybinds()

    CONFIGS["list_win"] = list_win
    CONFIGS["log_win"] = log_win
    CONFIGS["log_win_main"] = log_win_main
    CONFIGS["input_win"] = input_win

    # special "config" for main screen/window
    CONFIGS["screen"] = screen

    logger.debug("initialized window configurations")


def _get_conversation_config() -> Dict[str, str]:
    """
    Initialize/get conversation configuration
    """

    # init configuration from defaults
    conversation_config = DEFAULT_CONVERSATION_CONFIG

    # parse config read from file
    config = read_from_file()
    for section in config.sections():
        if section == "conversations":
            # overwrite default keymap config entries
            for key in config["conversations"]:
                if key in conversation_config:
                    conversation_config[key] = config["conversations"][key]

    # write (updated) config to file again
    config["conversations"] = conversation_config
    write_to_file(config)

    # return config
    return conversation_config


def init_conversation_settings() -> None:
    """
    Initialize conversation settings
    """

    settings = _get_conversation_config()
    CONFIGS["conversations"] = settings
    logger.debug("initialized conversation settings")


def init_path() -> None:
    """
    Initialize configuration path. Make sure config directories exist.
    """

    config_path = Path.home() / ".config/nuqql"
    Path(config_path).mkdir(parents=True, exist_ok=True)
    logger.debug("initialized config path %s", config_path)


def init(screen: Any) -> None:
    """
    Initialize configurations
    """

    logger.debug("initializing config with screen %s", screen)
    init_path()
    init_win(screen)
    init_conversation_settings()


def init_logging() -> None:
    """
    Initialize logging
    """

    # does not go to the log file
    logger.debug("initializing logging")

    init_path()

    # map loglevel
    loglevel_map = {
        "":         logging.WARNING,    # default loglevel
        "debug":    logging.DEBUG,
        "info":     logging.INFO,
        "warn":     logging.WARNING,
        "error":    logging.ERROR,
    }
    loglevel = loglevel_map[get("loglevel")]

    # configure logging
    fmt = "%(asctime)s - %(levelname)-5.5s - %(name)s - %(message)s"
    formatter = logging.Formatter(fmt=fmt)

    file_name = Path.home() / ".config/nuqql" / "nuqql.log"
    fileh = logging.FileHandler(file_name)
    fileh.setLevel(loglevel)
    fileh.setFormatter(formatter)

    main_logger = logging.getLogger("nuqql")
    main_logger.propagate = False
    main_logger.setLevel(loglevel)
    main_logger.addHandler(fileh)

    # restrict log file access
    os.chmod(file_name, stat.S_IRUSR | stat.S_IWUSR)

    # log debug message
    main_logger.debug("logging initialized")


def parse_args() -> None:
    """
    Parse command line arguments.
    """

    # does not go to the log file
    logger.debug("parsing command line arguments")

    # if we add more, consider moving it to config or somewhere else
    parser = argparse.ArgumentParser(
        description="Run nuqql command line instant messenger.")
    parser.add_argument("--version", action="version", version=VERSION)
    parser.add_argument("--loglevel", choices=["debug", "info", "warn",
                                               "error"],
                        help="Logging level")
    parser.add_argument("--escdelay", default="100",
                        help="ESC delay for curses")
    args = parser.parse_args()

    # configure loglevel and logging
    CONFIGS["loglevel"] = ""
    if args.loglevel:
        CONFIGS["loglevel"] = args.loglevel
    init_logging()

    # configure esc delay
    CONFIGS["escdelay"] = args.escdelay
